#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (C) 2006 by Vermeulen Adrien <adrien.vermeulen@free.fr>
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the
# Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

''' Standart Python Modules '''
import os
import re
import sys

''' PyQt and PyKDE Modules'''
from qt import *
from kdecore import KAboutData, KCmdLineArgs, KApplication, KProcess, KWinModule, KWin
from kfile import KFileDialog
from kdeui import QXEmbed
import ksnes_ui

home = os.popen("echo $HOME").read()[:-1] + "/"

def getWindow(pid):
	""" return a window info object for the process id (or None)
	
	"""
	for winid in KWinModule().windows():
		info = KWin.info(winid)
		if info.name.find("Snes9X") != -1: #the only way I found to catch the right window and yes it doesn't use the pid
			return info

class MainDialog(ksnes_ui.MainWindow):
	
	def __init__(self, *args):
		apply(ksnes_ui.MainWindow.__init__, (self,) + args)
		
		self.binPath = "/home/adrien/Snes/snes9x-1.5-src/snes9x" #my path to snes9x
		
		self.snesProcess = QProcess(self, "snes")
		self.snesProcess.addArgument(self.binPath)
		
		#self.embeded.setProtocol(QXEmbed.XPLAIN)
		#self.view.addWidget(self.embeded, 12)
		#embeded = self.embeded
		#stack.addWidget(embeded)
		#self.view.raiseWidget(self.embeded)
		
		self.options = []
		
		self.playButton.setEnabled(False)
		self.stopButton.setEnabled(False)
		
		self.connect(self.openButton, SIGNAL("released()"), self.openClicked)
		self.connect(self.stopButton, SIGNAL("released()"), self.stopClicked)
		self.connect(self.playButton, SIGNAL("released()"), self.playClicked)
		self.connect(self.resetButton, SIGNAL("released()"), self.resetClicked)
		self.connect(self.configButton, SIGNAL("released()"), self.configClicked)
		self.connect(self.fsButton, SIGNAL("released()"), self.fsClicked)
		self.connect(self.aboutButton, SIGNAL("released()"), self.aboutClicked)
	
	def openClicked(self):
		filename = KFileDialog.getOpenFileName(home + "Snes/roms", "", self, "Select a snes rom to open")
		self.launchRom(filename)
	
	def stopClicked(self):
		if (self.snesProcess.isRunning()):
			self.snesProcess.tryTerminate()
			self.snesProcess.clearArguments()
			self.snesProcess.addArgument(self.binPath)
	
	def playClicked(self):
		pass
	
	def resetClicked(self):
		#self.view.raiseWidget(12)
		pass
	
	def configClicked(self):
		pass
	
	def fsClicked(self):
		if self.isFullScreen():
			self.setWindowState(QWidget.WindowNoState)
		else:
			self.setWindowState(QWidget.WindowFullScreen)
	
	def aboutClicked(self):
		pass
	
	def launchRom(self, romfile):
		self.snesProcess.addArgument(romfile)
		code = self.snesProcess.start()
		if code:
			pid = self.snesProcess.processIdentifier()
			self.launchPid = pid
			QTimer.singleShot(2000, self.embedLaunchedWindow)
		else:
			print 'failed to start %s' % romfile
	
	def embedLaunchedWindow(self):
		winobj = getWindow(self.launchPid)
		if winobj:
			stack = self.view # the widget stack in wich I want to embed snes9x
			embeded = QXEmbed(self)
			stack.addWidget(embeded,12)
			embeded.embed(winobj.win)
			stack.raiseWidget(embeded)
			self.stopButton.setEnabled(True)

#-------------------- main ------------------------------------------------
if __name__ == "__main__":
	description = "A frontend for snes9x 1.5"
	version     = "0.1"
	aboutData   = KAboutData ("KSnes","KSnes", version, description, KAboutData.License_GPL_V2, "(C) 2006 AdrienV")
	
	aboutData.addAuthor ("AdrienV", "", "adrien.vermeulen@free.fr")
	
	KCmdLineArgs.init (sys.argv, aboutData)
	
	KCmdLineArgs.addCmdLineOptions ([("+files", "File to open")])
	
	app = KApplication()
	app.connect(app,SIGNAL("lastWindowClosed()"),app,SLOT("quit()"))
	mainWindow = MainDialog()
	#app.setMainWindow(mainWindow)
	mainWindow.show()
	sys.exit(app.exec_loop())